from django.contrib import admin
from django import forms
from django_admin_listfilter_dropdown.filters import RelatedDropdownFilter
from django.core.exceptions import ValidationError

from apps.MySU.models import AssociationMembership


class AssociationMembershipForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['user'].required = True

    class Meta:
        model = AssociationMembership
        fields = '__all__'

    def clean(self):
        cleaned_data = super().clean()
        association = cleaned_data.get("association")
        user = cleaned_data.get("user")
        profile = cleaned_data.get("profile")
        type_a = cleaned_data.get("type")
        new_type = cleaned_data.get("new_type")
        change_type_on_period_end = cleaned_data.get("change_type_on_period_end")
        date_left = cleaned_data.get("date_left")
        date_joined = cleaned_data.get("date_joined")
        if user is not None:
            if user != profile.user:
                raise ValidationError("Profile must belong to the user")
        if association is not None:
            if type_a.association != association:
                raise ValidationError("Membertype must belong to the association")
        if new_type:
            if new_type.association != self.association:
                raise ValidationError("Membertype must belong to the association")
        if change_type_on_period_end and new_type is None:
            raise ValidationError("You cannot say you change your type, but not have a type to change to")
        if date_left:
            if date_left < date_joined:
                raise ValidationError(
                    {'date_left': "End date must be after start date."},
                )

@admin.register(AssociationMembership)
class AssociationMembershipAdmin(admin.ModelAdmin):
    form = AssociationMembershipForm
    list_display = (
        'slug',
        'user',
        'association',
        'profile',
        'type',
        'date_joined',
        'date_left',
        'status'
    )
    list_filter = (
        'status',
        ('association', RelatedDropdownFilter),
    )
    list_display_links = (
        'slug',
        'user',
        'association',
        'profile',
        'type',
    )

    search_fields = (
        'user__username',
        'association__name',
    )

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return (
                'user',
                'association',
            )
        else:
            return ()
