from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from apps.MySU.models import AssociationSpecificData
from apps.MySU.serializers.base import BaseSerializer
from MySU.helper_functions import get_for_patch



class AssociationSpecificDataSerializerSimple(serializers.HyperlinkedModelSerializer):
    name = serializers.ReadOnlyField(source='data_field.name')

    class Meta:
        model = AssociationSpecificData
        fields = ('data_field', 'name', 'value')
        extra_kwargs = {
            'data_field': {'lookup_field': 'slug'},
        }


class AssociationSpecificDataSerializer(BaseSerializer):
    name = serializers.ReadOnlyField(source='data_field.name')

    class Meta:
        model = AssociationSpecificData
        fields = ('url', 'slug', 'membership', 'association', 'data_field', 'name', 'value')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'data_field': {'lookup_field': 'slug'},
            'membership': {'lookup_field': 'slug'},
        }
        create_only_fields = ('association', 'data_field', 'membership')
        nested_fields = ['association', 'membership']

    def validate(self, attrs):
        attrs = super(AssociationSpecificDataSerializer, self).validate(attrs)
        association = get_for_patch(self.instance, attrs, 'association')
        data_field = get_for_patch(self.instance, attrs, 'data_field')
        membership = get_for_patch(self.instance, attrs, 'membership')
        value = get_for_patch(self.instance, attrs, 'value')

        errors = {}
        if association != data_field.association:
            errors["Association"] = f"{association.slug}, doesn't match the association, {data_field.association.slug}, of the data_field {data_field.slug}"
        if association != membership.association:
            errors["Association"] = f"{association.slug}, doesn't match the association, {data_field.association.slug}, of the membership {membership.slug}"
        if data_field.type == "Boolean" and data_field.mandatory and not value:
            errors["mandatory"] = "Value has to be true, when data field is mandatory."
        if errors:
            raise ValidationError(errors)
        return attrs
