from django.core.mail import send_mail
from rest_framework import permissions
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from apps.buddy.models import Matching
from apps.calendar.models import Enrollment
from apps.MySU.serializers.bulk_email import BulkEmailSerializer


class EmailEnrollmentsView(APIView):
    http_method_names = ['post']
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request, format=None):
        serializer = BulkEmailSerializer(data=request.data, context={"request": request})
        if serializer.is_valid():
            data = serializer.validated_data
            enrollments_for_association = Matching.objects.filter(association=data["association"])
            if data["sent_to_all"]:
                enrollments_to_be_emailed = enrollments_for_association
            else:
                enrollments_to_be_emailed = enrollments_for_association.filter(email_send=False)
            for enrollment in enrollments_to_be_emailed:
                enrollment.send_enroll_email(update_email_send_in_db=False)
                enrollment.email_send = True
            Enrollment.objects.bulk_update(enrollments_to_be_emailed, ["email_send"], 50)
            return Response({"emails send": len(enrollments_to_be_emailed)}, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
