from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import Profile
from apps.MySU.models import User
from tests.tests import fixtures


class ProfileTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_put_self(self):
        pass

    def test_put_other(self):
        pass

    def test_patch_self(self):
        response = self.client.get('/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c')
        self.assertEqual(200, response.status_code, "Error: {}".format(response.data))
        self.assertEqual(None, response.data["study"], response.data)
        data = {
            "study": "/studies/89ac22dc-5a71-4fac-8af4-5844b7eb71a1"
        }
        response = self.client.patch('/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c?expand=study', data=data)
        self.assertEqual(200, response.status_code, "Error: {}".format(response.data))
        self.assertEqual("Applied Physics", response.data["study"]["name"], response.data)
        self.assertEqual("http://testserver/studies/89ac22dc-5a71-4fac-8af4-5844b7eb71a1", response.data["study"]["url"], response.data)
        data = {
            "study": None
        }
        response = self.client.patch('/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c', data=data, format="json")
        self.assertEqual(200, response.status_code, "Error: {}".format(response.data))
        self.assertEqual(None, response.data["study"], response.data)
        self.assertEqual("NL15RABO6359906732", response.data["iban"], response.data)
        data = {
            "bank_account_name": "Fiets",
            "iban": "NL97ABNA7080876885",
            "bic": ""
        }
        response = self.client.patch('/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c', data=data, format="json")
        self.assertEqual(200, response.status_code, "Error: {}".format(response.data))
        self.assertEqual("NL97ABNA7080876885", response.data["iban"], response.data)
        self.assertEqual("Fiets", response.data["bank_account_name"], response.data)

    def test_patch_other(self):
        data = {
            "gender": Profile.Gender.MALE
        }
        response = self.client.patch('/profiles/925d833b-1eef-486f-a4b5-9f948d5b91c4', data=data)
        self.assertEqual(404, response.status_code, "Error: {}".format(response.data))

        response = self.client.patch('/profiles/83d82e6d-9e9c-4705-a803-49865f32eab7', data=data)
        self.assertEqual(403, response.status_code, "Error: {}".format(response.data))

        response = self.client.patch('/profiles/c75fdcc3-9fca-43f4-959f-cf257c4c3165', data=data)
        self.assertEqual(200, response.status_code, "Error: {}".format(response.data))
        self.assertEqual(Profile.Gender.MALE, response.data["gender"], response.data)

    def test_post_self(self):
        # Student number is included, but should be dropped
        data = {
            "profilename": "KAAS",
            "address": "Anonymised",
            "zip_code": "Anonymised",
            "city": "Anonymised",
            "country": "Anonymised",
            "slug": '925d833b-1eef-486f-a4b5-9f948d5b91c4',
            "user": 3,
            "student_number": "Anonymised",
            "given_name": "Anonymised",
            "first_names": "Anonymised",
            "surname": "Anonymised",
            "phone_number": "Anonymised",
            "iban": '123455678123s',
            "bank_account_name": "Anonymised"
        }
        response = self.client.post('/profiles', data=data)
        self.assertEqual(201, response.status_code, "Error: {}".format(response.data))
        # Student number should be set equal to the student number of the user creating it.
        self.assertEqual("s1234567", response.data["student_number"], response.data)
        self.assertEqual("123455678123s", response.data["iban"], response.data)
        self.assertEqual("", response.data["bic"], response.data)
        self.assertEqual("Anonymised", response.data["bank_account_name"], response.data)

    def test_post_other(self):
        # user cannot set user in post request; will be done based on authenticated user.
        profile_count_pre_post = Profile.objects.filter(user=2).count()
        data = {
            "profilename": "KAAS",
            "address": "Anonymised",
            "zip_code": "Anonymised",
            "city": "Anonymised",
            "country": "Anonymised",
            "slug": '925d833b-1eef-486f-a4b5-9f948d5b91c4',
            "user": 2,
            "student_number": "Anonymised",
            "given_name": "Anonymised",
            "initials": "A",
            "surname": "Anonymised",
            "phone_number": "Anonymised",
        }
        response = self.client.post('/profiles', data=data)
        self.assertEqual(201, response.status_code, "Error: {}".format(response.data))
        profile_count_post_post = Profile.objects.filter(user=2).count()
        self.assertEqual(profile_count_pre_post, profile_count_post_post, "Error: Number of profiles should not change")

    def test_get_other(self):
        pass

    def test_list(self):
        response = self.client.get('/profiles')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all profiles".format(response.data))
        self.assertEqual(6, response.data['count'], msg="number of profiles doesn't match expected number of profiles")

        mines = sum([profile["mine"] for profile in response.data['results']])
        self.assertEqual(1, mines, msg="Only 1 profile belongs to logged in user")
        # response = self.client.get('/profiles?mine=True')
        # self.assertEqual(200, response.status_code, "Error: {}, Input: List of all profiles".format(response.data))
        # self.assertEqual(1, response.data['count'], msg="number of profiles doesn't match expected number of profiles")
